<?php

namespace Modules\DF\Services;

use Modules\DF\Factories\Guia as Factories;

class Guia{
    
    protected $factory;
    protected $errors = array();
    public $guia_struct = array(
                                "DTINICIAL" => array(
                                                        "TYPE" => "string",
                                                        "SIZE" => "8",
                                                        "REQUIRED" => false
                                                        ),
                                "DTFINAL" => array(
                                                        "TYPE" => "string",
                                                        "SIZE" => "8",
                                                        "REQUIRED" => false
                                                        ),
                                "NRINSJURGUIA" => array(
                                                        "TYPE" => "string",
                                                        "SIZE" => "11",
                                                        "REQUIRED" => false
                                                        ),
                                "NRNOTAFISC" => array(
                                                        "TYPE" => "string",
                                                        "SIZE" => "4",
                                                        "REQUIRED" => false
                                                        ),
                                "CDFILIAL" => array(
                                                        "TYPE" => "string",
                                                        "SIZE" => "4",
                                                        "REQUIRED" => true
                                                        ),
                                "IDSTATUS" => array(
                                                        "TYPE" => "string",
                                                        "SIZE" => "1",
                                                        "REQUIRED" => false
                                                        )
                                );
                                
    public function __construct(Factories $factory) {
        $this->factory = $factory;
    }
    
    public function validateGuideFilter($query){
        $query_bkp = $query;
        foreach($this->guia_struct as $struct_name => $struct_value){
            if(array_key_exists($struct_name, $query)){
                $field_value = $query[$struct_name];
                //$this->verifyType($this->guia_struct, $struct_name, $field_value);
                //$this->verifySize($this->guia_struct, $struct_name, $field_value);
                $this->verifyRequired($this->guia_struct, $struct_name, $field_value);
                
                if($struct_name == 'DTINICIAL'){
                    $query_bkp[$struct_name] = $this->dateFormat($field_value, $struct_name);
                }
                
                if($struct_name == 'DTFINAL'){
                    $query_bkp[$struct_name] = $this->dateFormat($field_value, $struct_name);
                }
                
                 if($struct_name == 'NRINSJURGUIA'){
                    if($query[$struct_name] == ""){
                        $query_bkp[$struct_name] = NULL;
                    }
                }
                
                if($struct_name == 'NRNOTAFISC'){
                    if($query[$struct_name] == ""){
                        $query_bkp[$struct_name] = NULL;
                    }
                }
                
            }else{
                $this->verifyRequired($this->guia_struct, $struct_name);
                if($struct_name == 'DTINICIAL'){
                    if(!array_key_exists($struct_name, $query)){
                        $query_bkp[$struct_name] = 'T';
                    }
                }
                
                if($struct_name == 'DTFINAL'){
                    if(!array_key_exists($struct_name, $query)){
                        $query_bkp[$struct_name] = date('d/m/Y');
                    }
                }
                
                if($struct_name == 'NRINSJURGUIA'){
                    if(!array_key_exists($struct_name, $query)){
                        $query_bkp[$struct_name] = NULL;
                    }
                }
                
                if($struct_name == 'NRNOTAFISC'){
                    if(!array_key_exists($struct_name, $query)){
                        $query_bkp[$struct_name] = NULL;
                    }
                }
                
                if($struct_name == 'IDSTATUS'){
                    if(!array_key_exists($struct_name, $query)){
                        $query_bkp[$struct_name] = 'T';
                    }
                }
            }
            if(sizeof($this->errors) > 0){
                return $this->returnValidation($this->errors, $query_bkp);
            }
        }
        return $this->returnValidation($this->errors, $query_bkp);
    }
    
    public function returnValidation($messages = array(), $guide = null){
        return array(
                    "errors" => $messages,
                    "guideUpdated" => $guide
                    );
    }
    
    public function dateFormat($field_value, $struct_name){
        $field_valueExploded = explode('/', $field_value);
        if(sizeof($field_valueExploded) != 3){
            array_push($this->errors, 'O ' .$struct_name. ' não corresponde ao formato de data especificado.');
        }else{
            $dtComHora = explode(' ', $field_valueExploded[2]);
            if(sizeof($dtComHora) > 1){
                $field_valueExploded[2] = $dtComHora[0];
            }
            $field_value = date_create("{$field_valueExploded[0]}-{$field_valueExploded[1]}-{$field_valueExploded[2]}");
            if($field_value != false){
                return $field_value->format('d/m/Y');
            }else{
                array_push($this->errors, "Formato da data incompatível");
                return 'T';
            }
        }
    }
    
    public function verifyType($struct, $field_struct_name, $field_value){
        if($struct[$field_struct_name]["TYPE"] != gettype($field_value)){
            array_push($this->errors, 'O atributo ' .$field_struct_name. ' não corresponde ao tipo de variável especificado. Tipo recebido: ' .gettype($field_value). '.'. ' Tipo requerido: '. $struct[$field_struct_name]["TYPE"] . '. Valor do atributo: ' . $field_value . '.');
        }
    }
    
    public function verifySize($struct, $field_struct_name, $field_value){
        if($struct[$field_struct_name]["SIZE"] != strlen($field_value)){
            $tamOrig = strlen($field_value);
            $field_value = $field_value == '' ? 'vazio' : $field_value;
            array_push($this->errors, 'O atributo ' .$field_struct_name. ' está com o tamanho incorreto. Tamanho recebido: ' .$tamOrig. '. Tamanho requerido: '. $struct[$field_struct_name]["SIZE"] . '. Valor do atributo: ' . $field_value . '.');
        } 
        
    }
    
    public function verifyRequired($struct, $field_struct_name, $field_value = null){
        if($struct[$field_struct_name]["REQUIRED"] == true){
            if($field_value == '' || !isset($field_value)){
                $msg = 'O atributo ' .$field_struct_name. ' é obrigatório e não foi informado. Valor recebido: ' .$field_value. '.';
                array_push($this->errors, $msg);
            } 
        }
    }

    public function getGuides($query){
        $taxes_bkp = array();
        $taxes = $this->factory->getGuides($query["DTINICIAL"], $query["DTFINAL"], $query["NRINSJURGUIA"], $query["NRNOTAFISC"], $query["CDFILIAL"]);
        if($taxes == false){
            array_push($this->errors, 'Nenhuma nota encontrada com esses parâmetros.');
            return array("taxes" => [], "errors" => $this->errors); 
        }else{
            if($query["IDSTATUS"] == "T"){
                foreach($taxes as $tax){
                        $tax["IDSTATUS"] = $this->factory->getIdStatus($tax["FEBOSID"]);
                        $tax["Produtos"] = $this->factory->getProducts($tax["CDFILIAL"], $tax["NRLANCTONF"]);
                        array_push($taxes_bkp, $tax);
                }
            }else{
                foreach($taxes as $tax){
                    $tax["IDSTATUS"] = $this->factory->getIdStatus($tax["FEBOSID"]);
                    if($tax["IDSTATUS"] == $query["IDSTATUS"]){
                        $tax["Produtos"] = $this->factory->getProducts($tax["CDFILIAL"], $tax["NRLANCTONF"]);
                        array_push($taxes_bkp, $tax);
                    }
                }
            }
        }
        
        if(sizeof($taxes_bkp) == 0){
            array_push($this->errors, 'Nenhuma nota encontrada com esses parâmetros.');
        }
        
        return array("taxes" => $taxes_bkp, "errors" => $this->errors); 
    }
}
